@@ -55,11 +55,6 @@ class ALS(Base, EvalMixin):
         self.seed = seed
         self.n_users = data_info.n_users
         self.n_items = data_info.n_items
-        self.default_prediction = (
-            data_info.global_mean
-            if task == "rating"
-            else 0.0
-        )
         self.user_consumed = data_info.user_consumed
         self.user_embed = None
         self.item_embed = None
@@ -103,16 +98,7 @@ class ALS(Base, EvalMixin):
                 print("="*30)
 
     def _choose_algo(self, use_cg):
-        if self.task == "rating":
-            if use_cg:
-                trainer = partial(als_update, task="rating", use_cg=True)
-            else:
-                trainer = partial(als_update, task="rating", use_cg=False)
-        elif self.task == "ranking":
-            if use_cg:
-                trainer = partial(als_update, task="ranking", use_cg=True)
-            else:
-                trainer = partial(als_update, task="ranking", use_cg=False)
+        trainer = partial(als_update, task=self.task, use_cg=use_cg)
         return trainer
 
     def predict(self, user, item):
