@@ -6,7 +6,6 @@ Reference: Weiping Song et al. "AutoInt: Automatic Feature Interaction Learning
 author: massquantity
 
 """
-import time
 from itertools import islice
 import numpy as np
 import tensorflow as tf2
@@ -20,7 +19,7 @@ from ..utils.tf_ops import (
 )
 from ..data.data_generator import DataGenFeat
 from ..utils.sampling import NegativeSampling
-from ..utils.unique_features import (
+from ..feature import (
     get_predict_indices_and_values,
     get_recommend_indices_and_values
 )
@@ -76,8 +75,6 @@ class AutoInt(Base, TfMixin, EvalMixin):
         self.n_users = data_info.n_users
         self.n_items = data_info.n_items
         self.global_mean = data_info.global_mean
-        self.default_prediction = (data_info.global_mean if (
-                task == "rating") else 0.0)
         self.seed = seed
         self.user_consumed = data_info.user_consumed
         self.sparse = self._decide_sparse_indices(data_info)
