@@ -26,6 +26,7 @@ class Base(abc.ABC):
     def __init__(self, task, data_info, lower_upper_bound=None):
         self.task = task
         if task == "rating":
+            self.global_mean = data_info.global_mean
             if lower_upper_bound is not None:
                 assert isinstance(lower_upper_bound, (list, tuple)), (
                     "must contain both lower and upper bound if provided")
@@ -39,6 +40,12 @@ class Base(abc.ABC):
         elif task != "ranking":
             raise ValueError("task must either be rating or ranking")
 
+        self.default_prediction = (
+            data_info.global_mean
+            if task == "rating"
+            else 0.0
+        )
+
     @abc.abstractmethod
     def fit(self, train_data, **kwargs):
         """Train model on the training data.
@@ -90,9 +97,11 @@ class Base(abc.ABC):
 
     def _check_unknown(self, user, item):
         unknown_user_indices = list(
-            np.where(np.logical_or(user >= self.n_users, user < 0))[0])
+            np.where(np.logical_or(user >= self.n_users, user < 0))[0]
+        )
         unknown_item_indices = list(
-            np.where(np.logical_or(item >= self.n_items, item < 0))[0])
+            np.where(np.logical_or(item >= self.n_items, item < 0))[0]
+        )
 
         unknown_user = (list(user[unknown_user_indices])
                         if unknown_user_indices
