@@ -13,7 +13,6 @@ from itertools import islice
 import numpy as np
 import tensorflow as tf2
 from tensorflow.keras.initializers import (
-    zeros as tf_zeros,
     truncated_normal as tf_truncated_normal
 )
 from .base import Base, TfMixin
@@ -25,7 +24,7 @@ from ..utils.tf_ops import (
 )
 from ..data.data_generator import DataGenFeat
 from ..utils.sampling import NegativeSampling
-from ..utils.unique_features import (
+from ..feature import (
     get_predict_indices_and_values,
     get_recommend_indices_and_values
 )
@@ -74,9 +73,6 @@ class FM(Base, TfMixin, EvalMixin):
         self.batch_sampling = batch_sampling
         self.n_users = data_info.n_users
         self.n_items = data_info.n_items
-        self.global_mean = data_info.global_mean
-        self.default_prediction = data_info.global_mean if (
-                task == "rating") else 0.0
         self.seed = seed
         self.user_consumed = data_info.user_consumed
         self.sparse = self._decide_sparse_indices(data_info)
