@@ -9,7 +9,6 @@ from itertools import islice
 import numpy as np
 import tensorflow as tf2
 from tensorflow.keras.initializers import (
-    zeros as tf_zeros,
     truncated_normal as tf_truncated_normal
 )
 from .base import Base, TfMixin
@@ -24,7 +23,7 @@ from ..utils.tf_ops import (
 from ..data.data_generator import DataGenFeat
 from ..utils.misc import colorize
 from ..utils.sampling import NegativeSampling
-from ..utils.unique_features import (
+from ..feature import (
     get_predict_indices_and_values,
     get_recommend_indices_and_values
 )
@@ -76,9 +75,6 @@ class WideDeep(Base, TfMixin, EvalMixin):
         self.hidden_units = list(map(int, hidden_units.split(",")))
         self.n_users = data_info.n_users
         self.n_items = data_info.n_items
-        self.global_mean = data_info.global_mean
-        self.default_prediction = data_info.global_mean if (
-                task == "rating") else 0.0
         self.seed = seed
         self.user_consumed = data_info.user_consumed
         self.sparse = self._decide_sparse_indices(data_info)
